/*
 * Copyright (c) 2020 The Linux Foundation. All rights reserved.
 *
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * DOC: Implement mgmt txrx APIs which shall be used internally only
 * in pkt_capture component.
 * Note: These APIs should be never accessed out of pkt_capture component.
 */

#include "wlan_pkt_capture_main.h"
#include "wlan_pkt_capture_priv.h"
#include "wlan_pkt_capture_mgmt_txrx.h"
#include "wlan_mlme_main.h"
#include "wlan_lmac_if_api.h"
#include "wlan_mgmt_txrx_utils_api.h"
#include "wlan_utility.h"
#include "cds_ieee80211_common.h"

/**
 * pkt_capture_mgmtpkt_cb() - callback to process management packets
 * for pkt capture mode
 * @context: vdev handler
 * @nbuf_list: netbuf list
 * @vdev_id: vdev id for which packet is captured
 * @tid:  tid number
 * @status: Tx status
 * @pkt_format: Frame format
 *
 * Return: none
 */
static void
pkt_capture_mgmtpkt_cb(void *context, void *nbuf_list,
		       uint8_t vdev_id, uint8_t tid, uint8_t status,
		       bool pkt_format)
{
	struct pkt_capture_vdev_priv *vdev_priv;
	struct wlan_objmgr_psoc *psoc = context;
	struct pkt_capture_cb_context *cb_ctx;
	struct wlan_objmgr_vdev *vdev;
	qdf_nbuf_t msdu, next_buf;
	uint32_t drop_count;

	vdev = wlan_objmgr_get_vdev_by_opmode_from_psoc(psoc,
							QDF_STA_MODE,
							WLAN_PKT_CAPTURE_ID);
	if (!vdev) {
		pkt_capture_err("vdev is NULL");
		goto free_buf;
	}

	vdev_priv = pkt_capture_vdev_get_priv(vdev);
	if (!vdev_priv) {
		pkt_capture_err("packet capture vdev priv is NULL");
		goto release_vdev_ref;
	}

	cb_ctx = vdev_priv->cb_ctx;
	if (!cb_ctx || !cb_ctx->mon_cb || !cb_ctx->mon_ctx) {
		pkt_capture_err("mon cb params are NULL");
		goto release_vdev_ref;
	}

	msdu = nbuf_list;
	while (msdu) {
		next_buf = qdf_nbuf_queue_next(msdu);
		qdf_nbuf_set_next(msdu, NULL);   /* Add NULL terminator */
		if (QDF_STATUS_SUCCESS !=
		    cb_ctx->mon_cb(cb_ctx->mon_ctx, msdu)) {
			pkt_capture_err("Frame Rx to HDD failed");
			qdf_nbuf_free(msdu);
		}
		msdu = next_buf;
	}

	wlan_objmgr_vdev_release_ref(vdev, WLAN_PKT_CAPTURE_ID);
	return;

release_vdev_ref:
	wlan_objmgr_vdev_release_ref(vdev, WLAN_PKT_CAPTURE_ID);
free_buf:
	drop_count = pkt_capture_drop_nbuf_list(nbuf_list);
	pkt_capture_debug("Dropped frames %u", drop_count);
}

/**
 * pkt_capture_mgmtpkt_process() - process management packets
 * for pkt capture mode
 * @psoc: pointer to psoc object
 * @txrx_status: mon_rx_status to update radiotap header
 * @nbuf: netbuf
 * @status: Tx status
 *
 * Return: QDF_STATUS Enumeration
 */
static QDF_STATUS
pkt_capture_mgmtpkt_process(struct wlan_objmgr_psoc *psoc,
			    struct mon_rx_status *txrx_status,
			    qdf_nbuf_t nbuf, uint8_t status)
{
	struct wlan_objmgr_vdev *vdev;
	struct pkt_capture_mon_pkt *pkt;
	uint32_t headroom;

	vdev = wlan_objmgr_get_vdev_by_opmode_from_psoc(psoc,
							QDF_STA_MODE,
							WLAN_PKT_CAPTURE_ID);
	if (!vdev) {
		pkt_capture_err("vdev is NULL");
		return QDF_STATUS_E_FAILURE;
	}

	/*
	 * Calculate the headroom and adjust head to prepare radiotap header
	 */
	headroom = qdf_nbuf_headroom(nbuf);
	qdf_nbuf_update_radiotap(txrx_status, nbuf, headroom);

	pkt = pkt_capture_alloc_mon_pkt(vdev);
	if (!pkt) {
		wlan_objmgr_vdev_release_ref(vdev, WLAN_PKT_CAPTURE_ID);
		return QDF_STATUS_E_FAILURE;
	}

	pkt->callback = pkt_capture_mgmtpkt_cb;
	pkt->context = psoc;
	pkt->monpkt = nbuf;
	pkt->vdev_id = WLAN_INVALID_VDEV_ID;
	pkt->tid = WLAN_INVALID_TID;
	pkt->status = status;
	pkt->pkt_format = PKTCAPTURE_PKT_FORMAT_80211;
	pkt_capture_indicate_monpkt(vdev, pkt);

	wlan_objmgr_vdev_release_ref(vdev, WLAN_PKT_CAPTURE_ID);
	return QDF_STATUS_SUCCESS;
}

QDF_STATUS
pkt_capture_process_mgmt_tx_data(struct wlan_objmgr_pdev *pdev,
				 struct mgmt_offload_event_params *params,
				 qdf_nbuf_t nbuf,
				 uint8_t status)
{
	struct mon_rx_status txrx_status = {0};
	struct wlan_objmgr_psoc *psoc;
	uint16_t channel_flags = 0;
	tpSirMacFrameCtl pfc = (tpSirMacFrameCtl) (qdf_nbuf_data(nbuf));
	struct ieee80211_frame *wh;
	uint8_t mgt_type;

	psoc = wlan_pdev_get_psoc(pdev);
	if (!psoc)
		return QDF_STATUS_E_FAILURE;

	wh = (struct ieee80211_frame *)qdf_nbuf_data(nbuf);
	mgt_type = (wh)->i_fc[0] & IEEE80211_FC0_TYPE_MASK;

	if (mgt_type == IEEE80211_FC0_TYPE_MGT &&
	    (pfc->subType == SIR_MAC_MGMT_DISASSOC ||
	     pfc->subType == SIR_MAC_MGMT_DEAUTH ||
	     pfc->subType == SIR_MAC_MGMT_ACTION)) {
		uint8_t *orig_hdr;
		uint8_t mic_len, hdr_len, pdev_id;

		if (!QDF_IS_ADDR_BROADCAST(wh->i_addr1) &&
		    !IEEE80211_IS_MULTICAST(wh->i_addr1)) {
			if (pfc->wep) {
				orig_hdr = (uint8_t *)qdf_nbuf_data(nbuf);
				pdev_id = wlan_objmgr_pdev_get_pdev_id(pdev);
				status = mlme_get_peer_mic_len(psoc, pdev_id,
							       wh->i_addr2,
							       &mic_len,
							       &hdr_len);
				if (QDF_IS_STATUS_ERROR(status)) {
					pkt_capture_err("Failed to get mic hdr");
					return QDF_STATUS_E_FAILURE;
				}

				/* Strip privacy headers (and trailer)
				 * for a received frame
				 */
				qdf_mem_move(orig_hdr +
					     hdr_len, wh,
					     sizeof(*wh));
				qdf_nbuf_pull_head(nbuf,
						   hdr_len);
				qdf_nbuf_trim_tail(nbuf, mic_len);
			}
		} else {
			qdf_nbuf_trim_tail(nbuf, IEEE80211_MMIE_LEN);
		}
	}

	txrx_status.tsft = (u_int64_t)params->tsf_l32;
	txrx_status.chan_num = wlan_freq_to_chan(params->chan_freq);
	txrx_status.chan_freq = params->chan_freq;
	/* params->rate is in Kbps, convert into Mbps */
	txrx_status.rate = (params->rate_kbps / 1000);
	if (params->rssi == INVALID_RSSI_FOR_TX)
		/* RSSI -128 is invalid rssi for TX, make it 0 here,
		 * will be normalized during radiotap updation
		 */
		txrx_status.ant_signal_db = 0;
	else
		txrx_status.ant_signal_db = params->rssi;

	txrx_status.rssi_comb = txrx_status.ant_signal_db;
	txrx_status.nr_ant = 1;
	txrx_status.rtap_flags |=
		((txrx_status.rate == 6 /* Mbps */) ? BIT(1) : 0);
	channel_flags |=
		((txrx_status.rate == 6 /* Mbps */) ?
		IEEE80211_CHAN_OFDM : IEEE80211_CHAN_CCK);
	channel_flags |=
		(wlan_reg_chan_to_band(txrx_status.chan_num) == BAND_2G ?
		IEEE80211_CHAN_2GHZ : IEEE80211_CHAN_5GHZ);
	txrx_status.chan_flags = channel_flags;
	txrx_status.rate = ((txrx_status.rate == 6 /* Mbps */) ? 0x0c : 0x02);

	wh = (struct ieee80211_frame *)qdf_nbuf_data(nbuf);
	wh->i_fc[1] &= ~IEEE80211_FC1_WEP;

	return pkt_capture_mgmtpkt_process(psoc, &txrx_status,
					   nbuf, status);
}

void
pkt_capture_mgmt_tx_completion(struct wlan_objmgr_pdev *pdev,
			       uint32_t desc_id,
			       uint32_t status,
			       struct mgmt_offload_event_params *params)
{
	qdf_nbuf_t wbuf, nbuf;
	int nbuf_len;

	if (!pdev) {
		pkt_capture_err("pdev is NULL");
		return;
	}

	nbuf = mgmt_txrx_get_nbuf(pdev, desc_id);
	if (!nbuf)
		return;

	nbuf_len = qdf_nbuf_len(nbuf);
	wbuf = qdf_nbuf_alloc(NULL, roundup(nbuf_len + RESERVE_BYTES, 4),
			      RESERVE_BYTES, 4, false);
	if (!wbuf) {
		pkt_capture_err("Failed to allocate wbuf for mgmt len(%u)",
				nbuf_len);
		return;
	}

	qdf_nbuf_put_tail(wbuf, nbuf_len);
	qdf_mem_copy(qdf_nbuf_data(wbuf), qdf_nbuf_data(nbuf), nbuf_len);

	if (QDF_STATUS_SUCCESS !=
		pkt_capture_process_mgmt_tx_data(pdev, params, wbuf, status))
		qdf_nbuf_free(wbuf);
}

/**
 * process_pktcapture_mgmt_rx_data_cb() -  process management rx packets
 * @rx_params: mgmt rx event params
 * @wbuf: netbuf
 *
 * Return: none
 */
static QDF_STATUS
pkt_capture_mgmt_rx_data_cb(struct wlan_objmgr_psoc *psoc,
			    struct wlan_objmgr_peer *peer,
			    qdf_nbuf_t wbuf,
			    struct mgmt_rx_event_params *rx_params,
			    enum mgmt_frame_type frm_type)
{
	struct mon_rx_status txrx_status = {0};
	uint16_t channel_flags = 0;
	struct ieee80211_frame *wh;
	qdf_nbuf_t nbuf;
	int buf_len;

	if (!(pkt_capture_get_mode(psoc) & PKT_CAPTURE_MODE_MGMT_ONLY))
		return QDF_STATUS_E_FAILURE;

	buf_len = qdf_nbuf_len(wbuf);
	nbuf = qdf_nbuf_alloc(NULL, roundup(
				  buf_len + RESERVE_BYTES, 4),
				  RESERVE_BYTES, 4, false);
	if (!nbuf)
		return QDF_STATUS_E_FAILURE;

	qdf_nbuf_put_tail(nbuf, buf_len);
	qdf_mem_copy(qdf_nbuf_data(nbuf), qdf_nbuf_data(wbuf), buf_len);

	txrx_status.tsft = (u_int64_t)rx_params->tsf_delta;
	txrx_status.chan_num = rx_params->channel;
	txrx_status.chan_freq = wlan_chan_to_freq(txrx_status.chan_num);
	/* rx_params->rate is in Kbps, convert into Mbps */
	txrx_status.rate = (rx_params->rate / 1000);
	txrx_status.ant_signal_db = rx_params->snr;
	txrx_status.rssi_comb = rx_params->snr;
	txrx_status.chan_noise_floor = NORMALIZED_TO_NOISE_FLOOR;
	txrx_status.nr_ant = 1;
	txrx_status.rtap_flags |=
		((txrx_status.rate == 6 /* Mbps */) ? BIT(1) : 0);
	channel_flags |=
		((txrx_status.rate == 6 /* Mbps */) ?
		IEEE80211_CHAN_OFDM : IEEE80211_CHAN_CCK);
	channel_flags |=
		(wlan_reg_chan_to_band(txrx_status.chan_num) == BAND_2G ?
		IEEE80211_CHAN_2GHZ : IEEE80211_CHAN_5GHZ);
	txrx_status.chan_flags = channel_flags;
	txrx_status.rate = ((txrx_status.rate == 6 /* Mbps */) ? 0x0c : 0x02);

	wh = (struct ieee80211_frame *)qdf_nbuf_data(nbuf);
	wh->i_fc[1] &= ~IEEE80211_FC1_WEP;

	if (QDF_STATUS_SUCCESS !=
		pkt_capture_mgmtpkt_process(psoc, &txrx_status, nbuf, 0))
		qdf_nbuf_free(nbuf);

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS pkt_capture_mgmt_rx_ops(struct wlan_objmgr_psoc *psoc,
				   bool is_register)
{
	struct mgmt_txrx_mgmt_frame_cb_info frm_cb_info;
	QDF_STATUS status;
	int num_of_entries;

	frm_cb_info.frm_type = MGMT_FRAME_TYPE_ALL;
	frm_cb_info.mgmt_rx_cb = pkt_capture_mgmt_rx_data_cb;
	num_of_entries = 1;

	if (is_register)
		status = wlan_mgmt_txrx_register_rx_cb(
					psoc, WLAN_UMAC_COMP_PKT_CAPTURE,
					&frm_cb_info, num_of_entries);
	else
		status = wlan_mgmt_txrx_deregister_rx_cb(
					psoc, WLAN_UMAC_COMP_PKT_CAPTURE,
					&frm_cb_info, num_of_entries);

	return status;
}
